/*
 * dma_bsp.h
 *
 *  Created on: Aug 9, 2023
 *      Author: hussamaldean
 */

#ifndef DMA_BSP_H_
#define DMA_BSP_H_

#include "stm32f4xx.h"
#include "stdint.h"


#define __BSP_DMA1__CLOCK_ENABLE()		RCC->AHB1ENR|=RCC_AHB1ENR_DMA1EN;
#define __BSP_DMA2__CLOCK_ENABLE()		RCC->AHB1ENR|=RCC_AHB1ENR_DMA2EN;



typedef enum
{
	dma1stream0=0,
	dma1stream1,
	dma1stream2,
	dma1stream3,
	dma1stream4,
	dma1stream5,
	dma1stream6,
	dma1stream7,

	dma2stream0,
	dma2stream1,
	dma2stream2,
	dma2stream3,
	dma2stream4,
	dma2stream5,
	dma2stream6,
	dma2stream7,

}DMAStreamTypedef;


typedef enum
{
	Channel0=0,
	Channel1,
	Channel2,
	Channel3,
	Channel4,
	Channel5,
	Channel6,
	Channel7
}DMA_Stream_ChannelTypedef;



typedef enum
{
	singleTransfer=0,
	INCR4,
	INCR8,
	INCR16,
}DMA_Stream_BurstTypedef;


typedef enum
{
	singleBuffer=0,
	doubleBuffer,
}DMA_StreamDoubleBufferTypedef;

typedef enum
{
	low=0,
	medium,
	high,
	very_high

}DMA_StreamPriorityTypedef;

typedef enum
{
	Psize=0,
	wordSize,
}DMA_StreamPrephIncSizeTypedef;


typedef enum
{
	byte=0,
	halfWord,
	word

}DMA_SizeTypedef;

typedef enum
{
	fixedMode=0,
	incMode=1
}DMA_IncModeTypedef;


typedef enum
{
	SingleTransfer=0,
	Circular=1
}DMA_CricularTypedef;


typedef enum
{
	peripheral2Memory=0,
	memory2Peripheral,
	memory2Memory

}DMA_DirectionTypedef;


typedef enum
{
	DMAController=0,
	PeripheralController,
}DMAPeripheralControlTypedef;


typedef enum
{
	InterruptDisabled=0,
	InterruptEnbaled=1,

	TransferComplet=1,
	HalfTransfer=1,
	TransferError=1,
	DirectModeError=1
}DMA_InterruptTypedef;

typedef struct
{

	uint8_t DMA_Channel;
	uint8_t PeripheralBurstMode;
	uint8_t MemoryBurstMode;
	uint8_t DoubleBuffer;
	uint8_t Priority;
	uint8_t PeripheralIncSize;
	uint8_t MemorySize;
	uint8_t PerpheralSize;
	uint8_t MemoryIncMode;
	uint8_t PeripheralIncMode;
	uint8_t CircularMode;
	uint8_t Direction;
	uint8_t PeripheralController;
	uint8_t InterruptEnable;
	uint8_t DMAStream;
	uint8_t TransferCompleteInterrupt;
	uint8_t HalfTransferInterrupt;
	uint8_t TransferErrorInterrupt;
	uint8_t DirectModeErrorInterrupt;
}DMA_ConfigTypedef;

void BSP_DMA_Init(DMA_Stream_TypeDef *dma, DMA_ConfigTypedef *config );

void BSP_DMA_Transfer_Single_Buffer(DMA_Stream_TypeDef *dma, uint32_t src, uint32_t dest, uint16_t len);

void BSP_DMA_Transfer_Double_Buffer(DMA_Stream_TypeDef *dma, uint32_t src, uint32_t dest1,uint32_t dest2, uint16_t len);


#endif /* DMA_BSP_H_ */
