/*
 * exti_bsp.c
 *
 *  Created on: Jul 20, 2023
 *      Author: hussamaldean
 */


#include "exti_bsp.h"
#include "stm32f4xx.h"


void bsp_exti_init(EXTI_ConfigTypedef *config)
{
	/*Disable global interrupts*/
	__disable_irq();

	/*Enable Clock Access to SYSCFG*/

	RCC->APB2ENR|=RCC_APB2ENR_SYSCFGEN;


	if (config->pinNumber>=0 &&config->pinNumber<=3)
	{
		SYSCFG->EXTICR[0] |= (config->port <<(config->pinNumber)*4);
	}

	if (config->pinNumber>=4 &&config->pinNumber<=7)
	{
		SYSCFG->EXTICR[1] |= (config->port <<(config->pinNumber-4)*4);
	}

	if (config->pinNumber>=8 &&config->pinNumber<=11)
	{
		SYSCFG->EXTICR[2] |= (config->port <<(config->pinNumber-8)*4);
	}

	if (config->pinNumber>=12 &&config->pinNumber<=15)
	{
		SYSCFG->EXTICR[3] |= (config->port <<(config->pinNumber-12)*4);
	}


	/*Unmask the interrupt line*/

	EXTI->IMR |=(1U<<config->pinNumber);

	switch (config->mode)
	{
		case RisingEdge:
			EXTI->RTSR|= (1U<<config->pinNumber);

			EXTI->FTSR&= ~(1U<<config->pinNumber);

			break;

		case FallingEdge:
			EXTI->RTSR&=~ (1U<<config->pinNumber);

			EXTI->FTSR|= (1U<<config->pinNumber);

			break;

		case Rising_FallingEdge:
			EXTI->RTSR|= (1U<<config->pinNumber);

			EXTI->FTSR|= (1U<<config->pinNumber);

			break;
	}

	/*Enable the interrupt in NVIC*/
	if(config->pinNumber==0)
	{
		NVIC_EnableIRQ(EXTI0_IRQn);
	}

	if(config->pinNumber==1)
	{
		NVIC_EnableIRQ(EXTI1_IRQn);
	}

	if(config->pinNumber==2)
	{
		NVIC_EnableIRQ(EXTI2_IRQn);
	}

	if(config->pinNumber==3)
	{
		NVIC_EnableIRQ(EXTI3_IRQn);
	}

	if(config->pinNumber==4)
	{
		NVIC_EnableIRQ(EXTI4_IRQn);
	}

	if(config->pinNumber>=5 && config->pinNumber<=9)
	{
		NVIC_EnableIRQ(EXTI9_5_IRQn);
	}

	if(config->pinNumber>=10 && config->pinNumber<=15)
	{
		NVIC_EnableIRQ(EXTI15_10_IRQn);
	}


	__enable_irq();

}

__WEAK void pin0_Callback(void)
{

}

__WEAK void pin1_Callback(void)
{

}

__WEAK void pin2_Callback(void)
{

}

__WEAK void pin3_Callback(void)
{

}

__WEAK void pin4_Callback(void)
{

}


__WEAK void pin5_Callback(void)
{

}

__WEAK void pin6_Callback(void)
{

}
__WEAK void pin7_Callback(void)
{

}

__WEAK void pin8_Callback(void)
{

}

__WEAK void pin9_Callback(void)
{

}
__WEAK void pin10_Callback(void)
{

}
__WEAK void pin11_Callback(void)
{

}
__WEAK void pin12_Callback(void)
{

}

__WEAK void pin13_Callback(void)
{

}
__WEAK void pin14_Callback(void)
{

}

__WEAK void pin15_Callback(void)
{

}




/*Interrupt handler*/

void EXTI0_IRQHandler (void)
{
	EXTI->PR |=(1<<pin0);
	pin0_Callback();
}


void EXTI1_IRQHandler (void)
{
	EXTI->PR |=(1<<pin1);
	pin1_Callback();
}

void EXTI2_IRQHandler (void)
{
	EXTI->PR |=(1<<pin2);
	pin2_Callback();
}

void EXTI3_IRQHandler (void)
{
	EXTI->PR |=(1<<pin3);
	pin3_Callback();
}

void EXTI4_IRQHandler (void)
{
	EXTI->PR |=(1<<pin4);
	pin4_Callback();
}


void EXTI9_5_IRQHandler(void)
{
	if (EXTI->PR & (1<<pin5))
	{
		EXTI->PR |=(1<<pin5);
		pin5_Callback();
	}

	if (EXTI->PR & (1<<pin6))
	{
		EXTI->PR |=(1<<pin6);
		pin5_Callback();

	}

	if (EXTI->PR & (1<<pin7))
	{
		EXTI->PR |=(1<<pin7);
		pin5_Callback();

	}

	if (EXTI->PR & (1<<pin8))
	{
		EXTI->PR |=(1<<pin8);
		pin5_Callback();

	}

	if (EXTI->PR & (1<<pin9))
	{
		EXTI->PR |=(1<<pin9);
		pin5_Callback();

	}

}

void EXTI15_10_IRQHandler(void)
{
	if (EXTI->PR & (1<<pin10))
	{
		EXTI->PR |=(1<<pin10);
		pin10_Callback();
	}

	if (EXTI->PR & (1<<pin11))
	{
		EXTI->PR |=(1<<pin11);
		pin11_Callback();
	}

	if (EXTI->PR & (1<<pin12))
	{
		EXTI->PR |=(1<<pin12);
		pin12_Callback();

	}

	if (EXTI->PR & (1<<pin13))
	{
		EXTI->PR |=(1<<pin13);
		pin13_Callback();

	}

	if (EXTI->PR & (1<<pin14))
	{
		EXTI->PR |=(1<<pin14);
		pin14_Callback();
	}

	if (EXTI->PR & (1<<pin15))
	{
		EXTI->PR |=(1<<pin15);
		pin15_Callback();
	}
}
