/*
 * spi_bsp.c
 *
 *  Created on: Aug 2, 2023
 *      Author: hussamaldean
 */


#include "spi_bsp.h"
#include "bsp.h"



void SPI_Enable_Clock(SPI_EnableTypedef spi_number)
{
	switch (spi_number)
	{
		case spi1: RCC->APB2ENR|=RCC_APB2ENR_SPI1EN;
			break;
		case spi2: RCC->APB1ENR|=RCC_APB1ENR_SPI2EN;
			break;
		case spi3: RCC->APB1ENR|=RCC_APB1ENR_SPI3EN;
			break;
		case spi4: RCC->APB2ENR|=RCC_APB2ENR_SPI4EN;
			break;
		case spi5: RCC->APB2ENR|=RCC_APB2ENR_SPI5EN;
			break;
		default :
			break;

	}
}

void BSP_SPI_Config(SPI_TypeDef *spi, SPI_ConfigTypedef *con)
{

	switch (con->mode)
	{

		 case Master_mode:  spi->CR1|=(1<<SPI_CR1_MSTR_Pos);
		 	 break;
		 case Slave_mode:   spi->CR1&=~(1<<SPI_CR1_MSTR_Pos); break;
		 default : break;
	}

	switch (con->dataFromat)
	{
		case MSB_First: spi->CR1&=~(con->dataFromat<<SPI_CR1_LSBFIRST_Pos);
			break;
		case LSB_First: spi->CR1|=(con->dataFromat<<SPI_CR1_LSBFIRST_Pos); break;
		default: break;
	}

	if(con->baud ==FCLK_2)
	{
		spi->CR1&=~(SPI_CR1_BR_Msk<<SPI_CR1_BR_Pos);
	}

	else
	{
		spi->CR1|=(con->baud<<SPI_CR1_BR_Pos);
	}



	if (con->CPOL==CPOL0)
	{
		spi->CR1&=~(1<<SPI_CR1_CPOL_Pos);
	}
	else
	{
		spi->CR1|=(con->CPOL<<SPI_CR1_CPOL_Pos);
	}

	if (con->CPHA==CPHA0)
	{
		spi->CR1&=~(1<<SPI_CR1_CPHA_Pos);
	}
	else
	{
		spi->CR1|=(con->CPHA<<SPI_CR1_CPHA_Pos);
	}



	switch (con->slaveManagement)
	{
	 case software_slave: spi->CR1|=(1<<SPI_CR1_SSM_Pos)|(1<<SPI_CR1_SSI_Pos); break;
	 case hardware_slave: spi->CR1&=~((1<<SPI_CR1_SSM_Pos)|(1<<SPI_CR1_SSI_Pos)); break;
	 default: break;
	}

	spi->CR1|=SPI_CR1_SPE;

}

SPI_StatusTypedef BSP_SPI_Transmit(SPI_TypeDef *spi ,uint8_t *data,uint32_t size, uint32_t timeout)
{

	uint32_t i=0;
	uint32_t start=BSP_Get_Ticks();
	while(i<size)
	{
		/*Wait until TXE is set*/
		while(!(spi->SR & (SPI_SR_TXE)))
		{
			if((BSP_Get_Ticks()-start)>timeout){return SPI_Timeout;}
		}

		/*Write the data to the data register*/
		spi->DR = data[i];
		i++;

	}
	/*Wait until TXE is set*/
	while(!(spi->SR & (SPI_SR_TXE))){if((BSP_Get_Ticks()-start)>timeout){return SPI_Timeout;}}

	/*Wait for BUSY flag to reset*/
	while((spi->SR & (SPI_SR_BSY))){if((BSP_Get_Ticks()-start)>timeout){return SPI_Timeout;}}

	/*Clear OVR flag*/
	(void)spi->DR;
	(void)spi->SR;

	return SPI_Succuss;
}


SPI_StatusTypedef BSP_SPI_Receive(SPI_TypeDef *spi ,uint8_t *data,uint32_t size, uint32_t timeout)
{
	uint32_t start=BSP_Get_Ticks();
	while(size)
	{
		/*Send dummy data*/
		spi->DR =0;

		/*Wait for RXNE flag to be set*/
		while(!(spi->SR & (SPI_SR_RXNE)))
		{
			if((BSP_Get_Ticks()-start)>timeout){return SPI_Timeout;}
		}

		/*Read data from data register*/
		*data++ = (spi->DR);
		size--;
	}
	return SPI_Succuss;
}
